/*
 * main.cpp
 */
#include <boost/bind.hpp>
#include <chores/DoDishesAction.h>
#include <chores/DoDishesFeedback.h>
#include <actionlib/client/simple_action_client.h>
#include <actionlib/server/simple_action_server.h>
#include <actionlib/client/simple_client_goal_state.h>

typedef actionlib::SimpleActionClient<chores::DoDishesAction> Client;
typedef actionlib::SimpleActionServer<chores::DoDishesAction> Server;

class ServerDude {
public:
  ServerDude()
    : server_(nh_, "do_dishes", boost::bind(&ServerDude::executeGoal, this, _1), false) {

    server_.start();
  }

private:
  ros::NodeHandle nh_;
  Server server_;

  void executeGoal(const chores::DoDishesGoalConstPtr& goal) {
    printf("Server called. Aborting goal.\n");
    server_.setAborted();
  }

};

class ClientGuy {
public:
  ClientGuy() : client_("do_dishes", true), goal_sent_(false) {

    client_.waitForServer();
  }

  void run() {
    while (ros::ok()) {
      if (!goal_sent_ || client_.getState().isDone()) {
        chores::DoDishesGoal goal;
        goal_sent_ = true;
        client_.sendGoal(goal,
                         boost::bind(&ClientGuy::doneCallback, this, _1, _2),
                         boost::bind(&ClientGuy::activeCallback, this),
                         boost::bind(&ClientGuy::feedbackCallback, this, _1));

        client_.waitForResult();
        ROS_INFO("Hogging time or something");

        printf("Current State: %s\n", client_.getState().toString().c_str());
      }
    }
  }

  void doneCallback(const actionlib::SimpleClientGoalState& state,
                    const chores::DoDishesResultConstPtr& result) {

    printf("Done.\n");
  }

  void activeCallback() {
    printf("Active.\n");
  }

  void feedbackCallback(const chores::DoDishesFeedbackConstPtr& feedback) {
    printf("Feedback.\n");
  }

private:
  Client client_;
  bool goal_sent_;

};

int main(int argc, char** argv) {
  ros::init(argc, argv, "test_client_server");

  ros::AsyncSpinner spinner(4);
  spinner.start();

  ServerDude sd;

  ClientGuy client;
  client.run();

  return 0;
}

